@@ -1,9 +1,7 @@
 #  Copyright (c) 2022 by Amplo.
 
-from amplo.automl.data_exploring import DataExplorer
 from amplo.automl.data_processing import DataProcessor
 from amplo.automl.data_sampling import DataSampler
-from amplo.automl.documenting import Documenter
 from amplo.automl.drift_detection import DriftDetector
 from amplo.automl.feature_processing import FeatureProcessor
 from amplo.automl.interval_analysis import IntervalAnalyser
