@@ -10,7 +10,7 @@ import time
 from datetime import datetime
 from inspect import signature
 from pathlib import Path
-from typing import Any, Tuple, Union
+from typing import Any
 from warnings import warn
 
 import joblib
@@ -23,7 +23,6 @@ from tqdm import tqdm
 
 import amplo
 from amplo import utils
-from amplo.automl.data_exploring import DataExplorer
 from amplo.automl.data_processing import DataProcessor
 from amplo.automl.data_sampling import DataSampler
 from amplo.automl.drift_detection import DriftDetector
@@ -35,6 +34,7 @@ from amplo.automl.feature_processing.feature_processor import (
 from amplo.automl.interval_analysis import IntervalAnalyser
 from amplo.automl.modelling import Modeller
 from amplo.automl.sequencing import Sequencer
+from amplo.base import BasePredictor
 from amplo.base.objects import LoggingMixin
 from amplo.classification.stacking import StackingClassifier
 from amplo.grid_search import ExhaustiveGridSearch, HalvingGridSearch, OptunaGridSearch
@@ -56,11 +56,11 @@ class Pipeline(LoggingMixin):
     ----------
     # Main parameters
     main_dir : str, default: "Auto_ML/"
-        Main directory of pipeline (for documentation).
+        Main directory of pipeline
     target : str, optional
         Column name of the output variable.
     name : str, default: "AutoML"
-        Name of the project (for documentation).
+        Name of the project
     version : int, default: 1
         Pipeline version. Will automatically increment when a version exists.
     mode : {None, "classification", "regression"}, default: None
@@ -71,16 +71,12 @@ class Pipeline(LoggingMixin):
         Default for regression: "mean_square_error".
     verbose : int, default: 1
         Verbosity of logging.
+    logging_to_file : bool, default: False
+        Whether to write logging to a file
+    logging_path : str, default: "AutoML.log"
+        Write to logging to given path if ``logs_to_file`` is True.
 
     # Data processing
-    int_cols : list of str, optional
-        Column names of integer columns (if None will detect automatically).
-    float_cols : list of str, optional
-        Column names of float columns (if None will detect automatically).
-    date_cols : list of str, optional
-        Column names of datetime column (if None will detect automatically).
-    cat_cols : list of str, optional
-        Column names of categorical columns (if None will detect automatically).
     missing_values : {"remove", "interpolate", "mean", "zero"}, default: "zero"
         How to treat missing values.
     outlier_removal : {"clip", "boxplot", "z-score", "none"}, default: "clip"
@@ -101,6 +97,8 @@ class Pipeline(LoggingMixin):
         Threshold for removing collinear features.
     feature_timeout : int, default: 3600
         Time budget for feature processing.
+    use_wavelets : bool, default: False
+        Whether to use wavelet transforms (useful for frequency data)
 
     # Interval analysis
     interval_analyse : bool, default: False
@@ -155,8 +153,6 @@ class Pipeline(LoggingMixin):
         Will be executed with `exec`, can be multi-line, uses data as input.
 
     # Flags
-    plot_eda : bool, default: False
-        Whether to run exploratory data analysis.
     process_data : bool, default: True
         Whether to force data processing.
     no_dirs : bool, default: False
@@ -171,18 +167,16 @@ class Pipeline(LoggingMixin):
         self,
         # Main settings
         main_dir: str = "Auto_ML/",
-        target: str | None = None,
+        target: str = "target",
         name: str = "AutoML",
-        version: int | None = None,
-        mode: str | None = None,
+        version: int = -1,
+        mode: str = "notset",
         objective: str | None = None,
         verbose: int = 1,
+        logging_to_file: bool = False,
+        logging_path: str = "AutoML.log",
         *,
         # Data processing
-        int_cols: list[str] | None = None,
-        float_cols: list[str] | None = None,
-        date_cols: list[str] | None = None,
-        cat_cols: list[str] | None = None,
         missing_values: str = "zero",
         outlier_removal: str = "clip",
         z_score_threshold: int = 4,
@@ -193,6 +187,7 @@ class Pipeline(LoggingMixin):
         extract_features: bool = True,
         information_threshold: float = 0.999,
         feature_timeout: int = 3600,
+        use_wavelets: bool = False,
         # Interval analysis
         interval_analyse: bool = True,
         # Sequencing
@@ -217,7 +212,6 @@ class Pipeline(LoggingMixin):
         # Production
         preprocess_function: str | None = None,
         # Flags
-        plot_eda: bool = False,
         process_data: bool = True,
         no_dirs: bool = False,
         # Other
@@ -230,11 +224,13 @@ class Pipeline(LoggingMixin):
         }
         del sig, init_locals
 
-        # super() calls
+        # Initialize Logger
         super().__init__(verbose=verbose)
+        if logging_to_file:
+            utils.logging.add_file_handler(logging_path)
 
         # Input checks: validity
-        if mode not in (None, "regression", "classification"):
+        if mode not in ("notset", "regression", "classification"):
             raise ValueError("Supported models: {'regression', 'classification', None}")
         if not 0 < information_threshold < 1:
             raise ValueError("Information threshold must be within (0, 1) interval.")
@@ -262,10 +258,6 @@ class Pipeline(LoggingMixin):
         self.objective = objective
 
         # Data processing
-        self.int_cols = int_cols
-        self.float_cols = float_cols
-        self.date_cols = date_cols
-        self.cat_cols = cat_cols
         self.missing_values = missing_values
         self.outlier_removal = outlier_removal
         self.z_score_threshold = z_score_threshold
@@ -278,6 +270,7 @@ class Pipeline(LoggingMixin):
         self.extract_features = extract_features
         self.information_threshold = information_threshold
         self.feature_timeout = feature_timeout
+        self.use_wavelets = use_wavelets
 
         # Interval analysis
         self.use_interval_analyser = interval_analyse
@@ -309,7 +302,6 @@ class Pipeline(LoggingMixin):
         self.preprocess_function = preprocess_function
 
         # Flags
-        self.plot_eda = plot_eda
         self.process_data = process_data
         self.no_dirs = no_dirs
 
@@ -333,26 +325,26 @@ class Pipeline(LoggingMixin):
 
         # Required sub-classes
         self.data_sampler = DataSampler()
-        self.data_processor = DataProcessor()
+        self.data_processor = DataProcessor(drop_datetime=True)
         self.data_sequencer = Sequencer()
         self.feature_processor = FeatureProcessor()
         self.interval_analyser = IntervalAnalyser()
         self.drift_detector = DriftDetector()
 
         # Instance initiating
-        self.best_model = None
-        self.best_model_str = None
-        self.best_feature_set = None
-        self.best_score = None
+        self.best_model: BasePredictor | None = None
+        self.best_model_str: str | None = None
+        self.best_feature_set: str | None = None
+        self.best_score: float | None = None
         self._data = None
-        self.feature_sets = None
-        self.results = None
-        self.n_classes = None
+        self.feature_sets: dict | None = None
+        self.results: pd.DataFrame | None = None
+        self.n_classes: int | None = None
         self.is_fitted = False
 
         # Monitoring
-        self._prediction_time = None
-        self._main_predictors = None
+        self._prediction_time: float | None = None
+        self._main_predictors: dict | None = None
 
     # User Pointing Functions
     def get_settings(self, version: int | None = None) -> dict:
@@ -372,7 +364,23 @@ class Pipeline(LoggingMixin):
             assert Path(
                 settings_path
             ).exists(), "Cannot load settings from nonexistent version"
-            return json.load(open(settings_path, "r"))
+            with open(settings_path, "r") as settings:
+                return json.load(settings)
+
+    def load(self):
+        """
+        Restores a pipeline from directory, given main_dir and version.
+        """
+        assert self.main_dir and self.version > -1
+
+        # Load settings
+        settings_path = self.main_dir + f"Production/v{self.version}/Settings.json"
+        with open(settings_path, "r") as settings:
+            self.load_settings(json.load(settings))
+
+        # Load model
+        model_path = self.main_dir + f"Production/v{self.version}/Model.joblib"
+        self.load_model(joblib.load(model_path))
 
     def load_settings(self, settings: dict):
         """
@@ -380,7 +388,8 @@ class Pipeline(LoggingMixin):
 
         Parameters
         ----------
-        settings [dict]: Pipeline settings
+        settings : dict
+            Pipeline settings.
         """
         # Set parameters
         settings["pipeline"]["no_dirs"] = True
@@ -391,12 +400,12 @@ class Pipeline(LoggingMixin):
         # TODO: load_settings for IntervalAnalyser (not yet implemented)
         if "drift_detector" in settings:
             self.drift_detector = DriftDetector(
-                num_cols=self.data_processor.float_cols + self.data_processor.int_cols,
+                num_cols=self.data_processor.num_cols,
                 cat_cols=self.data_processor.cat_cols,
                 date_cols=self.data_processor.date_cols,
             ).load_weights(settings["drift_detector"])
 
-    def load_model(self, model: object):
+    def load_model(self, model: BasePredictor):
         """
         Restores a trained model
         """
@@ -497,11 +506,7 @@ class Pipeline(LoggingMixin):
         self._data_processing()
 
         # Fit Drift Detector to input
-        num_cols = list(
-            set(self.x).intersection(
-                self.data_processor.float_cols + self.data_processor.int_cols
-            )
-        )
+        num_cols = list(set(self.x).intersection(self.data_processor.num_cols))
         date_cols = list(set(self.x).intersection(self.data_processor.date_cols))
         cat_cols = list(set(self.x) - set(num_cols + date_cols))
         self.drift_detector = DriftDetector(
@@ -509,9 +514,6 @@ class Pipeline(LoggingMixin):
         )
         self.drift_detector.fit(self.x)
 
-        # Run Exploratory Data Analysis
-        self._eda()
-
         # Balance data
         self._data_sampling()
 
@@ -540,7 +542,6 @@ class Pipeline(LoggingMixin):
         2. Grid Search
             Optimizes the hyperparameters of the best performing models
         3. (optional) Create Stacking model
-        4. (optional) Create documentation
 
         Parameters
         ----------
@@ -610,18 +611,19 @@ class Pipeline(LoggingMixin):
         # Finish
         self.is_fitted = True
         self.logger.info("All done :)")
-        utils.logging.remove_file_handler()
+        utils.logging.del_file_handlers()
 
     def convert_data(
         self, x: pd.DataFrame, preprocess: bool = True
-    ) -> Tuple[pd.DataFrame, pd.Series]:
+    ) -> tuple[pd.DataFrame, pd.Series | None]:
         """
         Function that uses the same process as the pipeline to clean data.
         Useful if pipeline is pickled for production
 
         Parameters
         ----------
-        data [pd.DataFrame]: Input features
+        data : pd.DataFrame
+            Input features.
         """
         # Convert to Pandas
         if isinstance(x, np.ndarray):
@@ -648,7 +650,8 @@ class Pipeline(LoggingMixin):
 
         # Sequence
         if self.sequence:
-            x, y = self.data_sequencer.convert(x, y)
+            self.logger.warn("Sequencer is temporarily disabled.", DeprecationWarning)
+            # x, y = self.data_sequencer.convert(x, y)
 
         # Convert Features
         x = self.feature_processor.transform(
@@ -657,13 +660,11 @@ class Pipeline(LoggingMixin):
 
         # Standardize
         if self.standardize:
+            assert isinstance(y, pd.Series)
             x, y = self._transform_standardize(x, y)
 
         # NaN test -- datetime should be taken care of by now
-        if (
-            x.astype("float32").replace([np.inf, -np.inf], np.nan).isna().sum().sum()
-            != 0
-        ):
+        if np.any(x.astype("float32").replace([np.inf, -np.inf], np.nan).isna()):
             raise ValueError(
                 f"Column(s) with NaN: {list(x.keys()[x.isna().sum() > 0])}"
             )
@@ -678,7 +679,8 @@ class Pipeline(LoggingMixin):
 
         Parameters
         ----------
-        data [pd.DataFrame]: data to do prediction on
+        data : pd.DataFrame
+            Data to do prediction on.
         """
         start_time = time.time()
         assert self.is_fitted, "Pipeline not yet fitted."
@@ -692,7 +694,10 @@ class Pipeline(LoggingMixin):
         x, y = self.convert_data(data)
 
         # Predict
+        assert self.best_model
         predictions = self.best_model.predict(x)
+
+        # Convert
         if self.mode == "regression" and self.standardize:
             predictions = self._inverse_standardize(predictions)
         elif self.mode == "classification":
@@ -714,7 +719,8 @@ class Pipeline(LoggingMixin):
 
         Parameters
         ----------
-        data [pd.DataFrame]: data to do prediction on
+        data : pd.DataFrame
+            Data to do prediction on.
         """
         start_time = time.time()
         assert self.is_fitted, "Pipeline not yet fitted."
@@ -784,9 +790,10 @@ class Pipeline(LoggingMixin):
         """
 
         # Allow target name to be set via __init__
-        target_name = utils.clean_feature_name(
+        target_name = (
             (target if isinstance(target, str) else None) or self.target or "target"
         )
+        clean_target_name = utils.clean_feature_name(target_name)
 
         # Read / set data
         if isinstance(data_or_path, (str, Path)):
@@ -809,8 +816,6 @@ class Pipeline(LoggingMixin):
         else:
             raise ValueError(f"Invalid type for `data_or_path`: {type(data_or_path)}")
 
-        data = utils.data.clean_keys(data)
-
         # Validate target
         if target is None or isinstance(target, str):
             if target_name not in data:
@@ -830,14 +835,27 @@ class Pipeline(LoggingMixin):
                 target.index = data.index
 
             if target_name in data:
-                raise ValueError(f"A '{target_name}' column already exists in `data`.")
-            data[target_name] = target
+                # Ignore when content is the same
+                if (data[target_name] != target).any():
+                    raise ValueError(
+                        f"The column '{target_name}' column already exists in `data` "
+                        f"but has different values."
+                    )
+            else:
+                data[target_name] = target
 
         else:
             raise ValueError("Invalid type for `target`.")
 
-        assert target_name in data, "Internal error: Target not in data."
-        self.target = target_name
+        # We clean the target but not the feature columns since the DataProcessor
+        # does not return the cleaned target name; just the clean feature columns.
+        if clean_target_name != target_name:
+            if clean_target_name in data:
+                msg = f"A '{clean_target_name}' column already exists in `data`."
+                raise ValueError(msg)
+            data = data.rename(columns={target_name: clean_target_name})
+        assert clean_target_name in data, "Internal error: Target not in data."
+        self.target = clean_target_name
 
         # Finish
         self._set_data(data)
@@ -873,7 +891,7 @@ class Pipeline(LoggingMixin):
         Detects the mode (Regression / Classification)
         """
         # Only run if mode is not provided
-        if self.mode is not None:
+        if self.mode in ("classification", "regression"):
             return
 
         # Classification if string
@@ -905,36 +923,35 @@ class Pipeline(LoggingMixin):
         """
         self.data_processor = DataProcessor(
             target=self.target,
-            int_cols=self.int_cols,
-            float_cols=self.float_cols,
-            date_cols=self.date_cols,
-            cat_cols=self.cat_cols,
             include_output=True,
+            drop_datetime=True,
             missing_values=self.missing_values,
             outlier_removal=self.outlier_removal,
             z_score_threshold=self.z_score_threshold,
         )
 
         # Set paths
-        extracted_data_path = self.main_dir + f"Data/Extracted_v{self.version}.csv"
-        data_path = self.main_dir + f"Data/Cleaned_v{self.version}.csv"
+        extracted_data_path = self.main_dir + f"Data/Extracted_v{self.version}.parquet"
+        data_path = self.main_dir + f"Data/Cleaned_v{self.version}.parquet"
         settings_path = self.main_dir + f"Settings/Cleaning_v{self.version}.json"
 
         # Only load settings if feature processor is done already.
         if Path(extracted_data_path).exists() and Path(settings_path).exists():
-            self.settings["data_processing"] = json.load(open(settings_path, "r"))
+            with open(settings_path, "r") as settings:
+                self.settings["data_processing"] = json.load(settings)
             self.data_processor.load_settings(self.settings["data_processing"])
+            return
 
         # Else, if data processor is done, load settings & data
         elif Path(data_path).exists() and Path(settings_path).exists():
             # Load data
-            data = self._read_csv(data_path)
+            data = self._read_df(data_path)
             self._set_data(data)
 
             # Load settings
-            self.settings["data_processing"] = json.load(open(settings_path, "r"))
+            with open(settings_path, "r") as settings:
+                self.settings["data_processing"] = json.load(settings)
             self.data_processor.load_settings(self.settings["data_processing"])
-
             self.logger.info("Loaded Cleaned Data")
 
         # Else, run the data processor
@@ -946,28 +963,23 @@ class Pipeline(LoggingMixin):
             self._set_data(data)
 
             # Store data
-            self._write_csv(self.data, data_path)
+            self._write_df(self.data, data_path)
 
             # Clean up Extracted previous version
             if os.path.exists(
-                self.main_dir + f"Data/Extracted_v{self.version - 1}.csv"
+                self.main_dir + f"Data/Extracted_v{self.version - 1}.parquet"
             ):
-                os.remove(self.main_dir + f"Data/Extracted_v{self.version - 1}.csv")
+                os.remove(self.main_dir + f"Data/Extracted_v{self.version - 1}.parquet")
 
             # Save settings
             self.settings["data_processing"] = self.data_processor.get_settings()
             if not self.no_dirs:
-                json.dump(self.settings["data_processing"], open(settings_path, "w"))
-
-        # If no columns were provided, load them from data processor
-        if self.date_cols is None:
-            self.date_cols = self.settings["data_processing"]["date_cols"]
-        if self.int_cols is None:
-            self.int_cols = self.settings["data_processing"]["int_cols"]
-        if self.float_cols is None:
-            self.float_cols = self.settings["data_processing"]["float_cols"]
-        if self.cat_cols is None:
-            self.cat_cols = self.settings["data_processing"]["cat_cols"]
+                with open(settings_path, "w") as settings:
+                    json.dump(
+                        self.settings["data_processing"],
+                        settings,
+                        cls=utils.io.NpEncoder,
+                    )
 
         # Assert classes in case of classification
         if self.mode == "classification":
@@ -978,21 +990,7 @@ class Pipeline(LoggingMixin):
                     "you may want to reconsider classification mode"
                 )
             if set(self.y) != set([i for i in range(len(set(self.y)))]):
-                raise ValueError("Classes should be [0, 1, ...]")
-
-    def _eda(self):
-        if not self.plot_eda:
-            return
-
-        self.logger.info("Starting Exploratory Data Analysis")
-        eda = DataExplorer(
-            self.x,
-            y=self.y,
-            mode=self.mode,
-            folder=self.main_dir,
-            version=self.version,
-        )
-        eda.run()
+                raise ValueError(f"Classes should be [0, 1, ...], not {set(self.y)}")
 
     def _data_sampling(self):
         """
@@ -1010,11 +1008,11 @@ class Pipeline(LoggingMixin):
             fast_run=False,
             objective=self.objective,
         )
-        data_path = self.main_dir + f"Data/Balanced_v{self.version}.csv"
+        data_path = self.main_dir + f"Data/Balanced_v{self.version}.parquet"
 
         if Path(data_path).exists():
             # Load data
-            data = self._read_csv(data_path)
+            data = self._read_df(data_path)
             self._set_data(data)
 
             self.logger.info("Loaded Balanced data")
@@ -1026,7 +1024,7 @@ class Pipeline(LoggingMixin):
 
             # Store
             self._set_xy(x, y)
-            self._write_csv(self.data, data_path)
+            self._write_df(self.data, data_path)
 
     def _sequencing(self):
         """
@@ -1042,11 +1040,11 @@ class Pipeline(LoggingMixin):
             shift=self.sequence_shift,
             diff=self.sequence_diff,
         )
-        data_path = self.main_dir + f"Data/Sequence_v{self.version}.csv"
+        data_path = self.main_dir + f"Data/Sequence_v{self.version}.parquet"
 
         if Path(data_path).exists():
             # Load data
-            data = self._read_csv(data_path)
+            data = self._read_df(data_path)
             self._set_data(data)
 
             self.logger.info("Loaded Extracted Features")
@@ -1058,7 +1056,7 @@ class Pipeline(LoggingMixin):
 
             # Store
             self._set_xy(x, y)
-            self._write_csv(self.data, data_path)
+            self._write_df(self.data, data_path)
 
     def _feature_processing(self):
         """
@@ -1068,22 +1066,24 @@ class Pipeline(LoggingMixin):
         self.feature_processor = FeatureProcessor(
             mode=self.mode,
             is_temporal=None,
+            use_wavelets=self.use_wavelets,
             extract_features=self.extract_features,
             collinear_threshold=self.information_threshold,
             verbose=self.verbose,
         )
 
         # Set paths
-        data_path = self.main_dir + f"Data/Extracted_v{self.version}.csv"
+        data_path = self.main_dir + f"Data/Extracted_v{self.version}.parquet"
         settings_path = self.main_dir + f"Settings/Extracting_v{self.version}.json"
 
         if Path(data_path).exists() and Path(settings_path).exists():
             # Loading data
-            data = self._read_csv(data_path)
+            data = self._read_df(data_path)
             self._set_data(data)
 
             # Loading settings
-            self.settings["feature_processing"] = json.load(open(settings_path, "r"))
+            with open(settings_path, "r") as settings:
+                self.settings["feature_processing"] = json.load(settings)
             self.feature_processor.load_settings(self.settings["feature_processing"])
             self.feature_sets = self.settings["feature_processing"]["feature_sets_"]
 
@@ -1100,16 +1100,21 @@ class Pipeline(LoggingMixin):
 
             # Store data
             self._set_xy(x, y)
-            self._write_csv(self.data, data_path)
+            self._write_df(self.data, data_path)
 
-            # Cleanup Cleaned_vx.csv
-            if os.path.exists(self.main_dir + f"Data/Cleaned_v{self.version}.csv"):
-                os.remove(self.main_dir + f"Data/Cleaned_v{self.version}.csv")
+            # Cleanup Cleaned_vx.parquet
+            if os.path.exists(self.main_dir + f"Data/Cleaned_v{self.version}.parquet"):
+                os.remove(self.main_dir + f"Data/Cleaned_v{self.version}.parquet")
 
             # Save settings
             self.settings["feature_processing"] = self.feature_processor.get_settings()
             if not self.no_dirs:
-                json.dump(self.settings["feature_processing"], open(settings_path, "w"))
+                with open(settings_path, "w") as settings:
+                    json.dump(
+                        self.settings["feature_processing"],
+                        settings,
+                        cls=utils.io.NpEncoder,
+                    )
 
     def _interval_analysis(self):
         """
@@ -1126,13 +1131,11 @@ class Pipeline(LoggingMixin):
         self.interval_analyser = IntervalAnalyser(target=self.target)
 
         # Set paths
-        data_path = self.main_dir + f"Data/Interval_Analyzed_v{self.version}.csv"
-        # settings_path = self.main_dir +
-        # f'Settings/Interval_Analysis_v{self.version}.json'
+        data_path = self.main_dir + f"Data/Interval_Analyzed_v{self.version}.parquet"
 
-        if Path(data_path).exists():  # TODO: and Path(settings_path).exists():
+        if Path(data_path).exists():
             # Load data
-            data = self._read_csv(data_path)
+            data = self._read_df(data_path)
             self._set_data(data)
 
             # TODO implement `IntervalAnalyser.load_settings` and add to
@@ -1151,13 +1154,13 @@ class Pipeline(LoggingMixin):
 
             # Store data
             self._set_data(data)
-            self._write_csv(self.data, data_path)
+            self._write_df(self.data, data_path)
 
             # TODO implement `IntervalAnalyser.get_settings` and add to
             #  `self.get_settings`
             # # Save settings
             # self.settings['interval_analysis'] = self.intervalAnalyser.get_settings()
-            # json.dump(self.settings['interval_analysis'], open(settings_path, 'w'))
+            # json.dump(self.settings['interval_analysis'], open(settings_path, 'w'), cls=utils.io.NpEncoder)
 
     def _standardizing(self):
         """
@@ -1172,14 +1175,18 @@ class Pipeline(LoggingMixin):
 
         if Path(settings_path).exists():
             # Load data
-            self.settings["standardize"] = json.load(open(settings_path, "r"))
+            with open(settings_path, "r") as settings:
+                self.settings["standardize"] = json.load(settings)
 
         else:
             # Fit data
             self._fit_standardize(self.x, self.y)
 
             # Store Settings
-            json.dump(self.settings["standardize"], open(settings_path, "w"))
+            with open(settings_path, "w") as settings:
+                json.dump(
+                    self.settings["standardize"], settings, cls=utils.io.NpEncoder
+                )
 
         # Transform data
         x, y = self._transform_standardize(self.x, self.y)
@@ -1222,6 +1229,7 @@ class Pipeline(LoggingMixin):
         if self.results is None or self.version not in self.results["version"].values:
 
             # Iterate through feature sets
+            assert self.feature_sets
             for feature_set, cols in self.feature_sets.items():
 
                 # Skip empty sets
@@ -1253,7 +1261,8 @@ class Pipeline(LoggingMixin):
                     self.results = pd.concat([self.results, results])
 
             # Save results
-            self._write_csv(self.results, results_path)
+            if not self.no_dirs:
+                self.results.to_csv(results_path)
 
     def grid_search(self, model=None, feature_set=None, parameter_set=None):
         """
@@ -1282,10 +1291,10 @@ class Pipeline(LoggingMixin):
         When both parameters, ``model`` and ``feature_set``, are provided, the grid
         search behaves as follows:
             - When both parameters are either of dtype ``str`` or have the same length,
-              then grid search will treat them as pairs.
+            then grid search will treat them as pairs.
             - When one parameter is an iterable and the other parameter is either a
-              string or an iterable of different length, then grid search will happen
-              for each unique combination of these parameters.
+            string or an iterable of different length, then grid search will happen
+            for each unique combination of these parameters.
         """
 
         # Skip grid search and set best initial model as best grid search parameters
@@ -1456,7 +1465,7 @@ class Pipeline(LoggingMixin):
                 "date": datetime.today().strftime("%d %b %y"),
                 "model": type(stack).__name__,
                 "dataset": feature_set,
-                "params": json.dumps(stack.get_params()),
+                "params": json.dumps(stack.get_params(), cls=utils.io.NpEncoder),
                 "mean_objective": np.mean(score),
                 "std_objective": np.std(score),
                 "mean_time": np.mean(times),
@@ -1651,7 +1660,8 @@ class Pipeline(LoggingMixin):
         self.settings["drift_detector"] = self.drift_detector.get_weights()
 
         # Save settings
-        json.dump(self.settings, open(settings_path, "w"), indent=4)
+        with open(settings_path, "w") as settings:
+            json.dump(self.settings, settings, indent=4, cls=utils.io.NpEncoder)
 
     # Getter Functions / Properties
     @property
@@ -1681,7 +1691,7 @@ class Pipeline(LoggingMixin):
             )
 
     @property
-    def data(self) -> Union[None, pd.DataFrame]:
+    def data(self) -> pd.DataFrame | None:
         return self._data
 
     @property
@@ -1715,15 +1725,15 @@ class Pipeline(LoggingMixin):
 
     def _set_xy(
         self,
-        new_x: Union[np.ndarray, pd.DataFrame],
-        new_y: Union[np.ndarray, pd.Series],
+        new_x: np.ndarray | pd.DataFrame,
+        new_y: np.ndarray | pd.Series,
     ):
         if not isinstance(new_y, pd.Series):
             new_y = pd.Series(new_y, name=self.target)
         new_data = pd.concat([new_x, new_y], axis=1)
         self._set_data(new_data)
 
-    def _set_x(self, new_x: Union[np.ndarray, pd.DataFrame]):
+    def _set_x(self, new_x: np.ndarray | pd.DataFrame):
         # Convert to dataframe
         if isinstance(new_x, np.ndarray):
             old_x = self.data.drop(self.target, axis=1)
@@ -1752,12 +1762,12 @@ class Pipeline(LoggingMixin):
         # Set data
         self._data = pd.concat([new_x, self.y], axis=1)
 
-    def set_y(self, new_y: Union[np.ndarray, pd.Series]):
+    def set_y(self, new_y: np.ndarray | pd.Series):
         self._data[self.target] = new_y
 
     # Support Functions
     @staticmethod
-    def _read_csv(data_path) -> pd.DataFrame:
+    def _read_df(data_path) -> pd.DataFrame:
         """
         Read data from given path and set index or multi-index
 
@@ -1765,23 +1775,11 @@ class Pipeline(LoggingMixin):
         ----------
         data_path : str or Path
         """
-        assert Path(data_path).suffix == ".csv", "Expected a *.csv path"
+        assert Path(data_path).suffix == ".parquet", "Expected a *.parquet path"
 
-        data = pd.read_csv(data_path)
+        return pd.read_parquet(data_path)
 
-        if {"index", "log"}.issubset(data.columns):
-            # Multi-index: case when IntervalAnalyser was used
-            index = ["log", "index"]
-        elif "index" in data.columns:
-            index = ["index"]
-        else:
-            raise IndexError(
-                "No known index was found. "
-                "Expected to find at least a column named `index`."
-            )
-        return data.set_index(index)
-
-    def _write_csv(self, data, data_path):
+    def _write_df(self, data, data_path):
         """
         Write data to given path and set index if needed.
 
@@ -1790,28 +1788,22 @@ class Pipeline(LoggingMixin):
         data : pd.DataFrame or pd.Series
         data_path : str or Path
         """
-        assert Path(data_path).suffix == ".csv", "Expected a *.csv path"
+        assert Path(data_path).suffix == ".parquet", "Expected a *.parquet path"
 
         # Set single-index if not already present
         if len(data.index.names) == 1 and data.index.name is None:
             data.index.name = "index"
 
-        # Raise error if unnamed index is present
-        if None in data.index.names:
-            raise IndexError(
-                f"Found an unnamed index column ({list(data.index.names)})."
-            )
-
         # Write data
         if not self.no_dirs:
-            data.to_csv(data_path)
+            data.to_parquet(data_path)
 
     def _load_version(self):
         """
         Upon start, loads version
         """
         # No need if version is set
-        if self.version is not None:
+        if self.version > -1:
             return
 
         # Find production folders
@@ -1823,10 +1815,7 @@ class Pipeline(LoggingMixin):
     def _create_dirs(self):
         folders = [
             "",
-            "EDA",
             "Data",
-            "Features",
-            "Documentation",
             "Production",
             "Settings",
         ]
@@ -1874,7 +1863,7 @@ class Pipeline(LoggingMixin):
 
     def _transform_standardize(
         self, x: pd.DataFrame, y: pd.Series
-    ) -> Tuple[pd.DataFrame, pd.Series]:
+    ) -> tuple[pd.DataFrame, pd.Series]:
         """
         Standardizes the input and output with values from settings.
 
@@ -1952,7 +1941,7 @@ class Pipeline(LoggingMixin):
         return utils.io.parse_json(results.iloc[0]["params"])
 
     def _grid_search_iteration(
-        self, model, parameter_set: Union[str, None], feature_set: str
+        self, model, parameter_set: str | None, feature_set: str
     ):
         """
         INTERNAL | Grid search for defined model, parameter set and feature set.
