@@ -1,21 +1,29 @@
+#  Copyright (c) 2022 by Amplo.
+
 import shutil
+import time
 from pathlib import Path
 
 import numpy as np
 import pandas as pd
 from sklearn.datasets import make_classification, make_regression
 
+from amplo.automl import Modeller
+
 __all__ = [
     "rmtree",
     "rmfile",
     "make_x_y",
     "make_data",
+    "get_all_modeller_models",
+    "find_first_warning_of_type",
     "RandomPredictor",
     "OverfitPredictor",
+    "DelayedRandomPredictor",
 ]
 
 
-def rmtree(folder="AutoML", must_exist=False):
+def rmtree(folder="Auto_ML", must_exist=False):
     if Path(folder).exists():
         shutil.rmtree(folder)
     elif must_exist:
@@ -47,10 +55,82 @@ def make_data(mode: str, target="target"):
     return data
 
 
+def get_all_modeller_models(mode: str, **kwargs):
+    models = {  # get each model type only once (!) with dictionary trick
+        type(model).__name__: model
+        for model in [
+            *Modeller(mode=mode, samples=100, **kwargs).return_models(),
+            *Modeller(mode=mode, samples=100_000, **kwargs).return_models(),
+        ]
+    }.values()
+    return list(models)
+
+
+def find_first_warning_of_type(typ, record):
+    """
+    Find first warning of type ``typ`` in warnings record.
+
+    Parameters
+    ----------
+    typ : Any
+        Warning to search for.
+    record : pytest.WarningsRecorder
+        Warnings record from ``with pytest.warns(...) as record`` context manager.
+
+    Returns
+    -------
+    pytest.WarningsMessage
+    """
+    for item in record:
+        if issubclass(item.category, typ):
+            return item
+    raise ValueError(f"No warning of type {typ} found in warnings record.")
+
+
 # ----------------------------------------------------------------------
 # Dummies
 
 
+class _DummyPredictor:
+    """
+    Dummy predictor for testing.
+
+    Parameters
+    ----------
+    mode : str
+        Predicting mode ("classification" or "regression").
+    """
+
+    _dummy_classifier = None
+    _dummy_regressor = None
+
+    def __init__(self, mode):
+        assert self._dummy_classifier is not None, "Dummy not set"
+        assert self._dummy_regressor is not None, "Dummy not set"
+
+        if mode == "classification":
+            self.predictor = self._dummy_classifier()
+        elif mode == "regression":
+            self.predictor = self._dummy_regressor()
+        else:
+            raise ValueError("Invalid predictor mode.")
+
+    def fit(self, x, y):
+        return self.predictor.fit(x, y)
+
+    def predict(self, x):
+        return self.predictor.predict(x)
+
+    def predict_proba(self, x):
+        assert isinstance(self.predictor, self._dummy_classifier)
+        return self.predictor.predict_proba(x)
+
+    @property
+    def classes_(self):
+        if hasattr(self.predictor, "classes"):
+            return self.predictor.classes
+
+
 class _RandomClassifier:
     """
     Dummy classifier for testing.
@@ -86,72 +166,9 @@ class _RandomRegressor:
         return np.random.uniform(*self.range, len(x))
 
 
-class RandomPredictor:
-    """
-    Dummy predictor for testing.
-
-    Parameters
-    ----------
-    mode : str
-        Predicting mode ("classification" or "regression").
-    """
-
-    def __init__(self, mode):
-        if mode == "classification":
-            self.predictor = _RandomClassifier()
-        elif mode == "regression":
-            self.predictor = _RandomRegressor()
-        else:
-            raise ValueError("Invalid predictor mode.")
-
-    def fit(self, x, y):
-        return self.predictor.fit(x, y)
-
-    def predict(self, x):
-        return self.predictor.predict(x)
-
-    def predict_proba(self, x):
-        assert isinstance(self.predictor, _RandomClassifier)
-        return self.predictor.predict_proba(x)
-
-    @property
-    def classes_(self):
-        if hasattr(self.predictor, "classes"):
-            return self.predictor.classes
-
-
-class OverfitPredictor:
-    """
-    Dummy predictor for testing.
-
-    Parameters
-    ----------
-    mode : str
-        Predicting mode ("classification" or "regression").
-    """
-
-    def __init__(self, mode):
-        if mode == "classification":
-            self.predictor = _OverfitClassifier()
-        elif mode == "regression":
-            self.predictor = _OverfitRegressor()
-        else:
-            raise ValueError("Invalid predictor mode.")
-
-    def fit(self, x, y):
-        return self.predictor.fit(x, y)
-
-    def predict(self, x):
-        return self.predictor.predict(x)
-
-    def predict_proba(self, x):
-        assert isinstance(self.predictor, _OverfitClassifier)
-        return self.predictor.predict_proba(x)
-
-    @property
-    def classes_(self):
-        if hasattr(self.predictor, "classes"):
-            return self.predictor.classes
+class RandomPredictor(_DummyPredictor):
+    _dummy_classifier = _RandomClassifier
+    _dummy_regressor = _RandomRegressor
 
 
 class _OverfitClassifier:
@@ -221,3 +238,22 @@ class _OverfitRegressor:
             else:
                 yt.append(self.y.iloc[ind[0]])
         return yt
+
+
+class OverfitPredictor(_DummyPredictor):
+    _dummy_classifier = _OverfitClassifier
+    _dummy_regressor = _OverfitRegressor
+
+
+class DelayedRandomPredictor(RandomPredictor):
+    def __init__(self, delay: float, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.delay = delay
+
+    def predict(self, x):
+        time.sleep(self.delay)
+        return super().predict(x)
+
+    def predict_proba(self, x):
+        time.sleep(self.delay)
+        return super().predict_proba(x)
